package com.example.service.serviceImpl;

import com.example.db1.dao.TscanAssortDao;
import com.example.db2.dao.LianZhongDao;
import com.example.service.CollectService;
import com.example.vo.CardInfoVo;
import com.example.vo.Picture;
import com.example.dto.TscanAssortDto;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.util.ObjectUtils;

import java.io.File;
import java.util.ArrayList;
import java.util.List;


/**
 * @ClassName CollectServiceImpl
 * @Description 同步联众数据
 * @Author linjj
 * @Date 2025/6/4 13:12
 * @Version 1.0
 */
@Service
@Slf4j
public class CollectServiceImpl implements CollectService {
    @Value("${savePath}")
    private String savePath;
    @Value("${collectFlag}")
    private Integer collectFlag;
    @Autowired
    private LianZhongDao lianZhongDao;
    @Autowired
    private TscanAssortDao tscanAssortDao;

    private final String uniUrl = "http://10.1.1.128";

    @Override
    public void LianZhongConllect() {
        //查询所有联众需要同步数据，未开始状态为0，正在同步为1，完成为2，失败为3，不存在文件为5
        Integer collectNum = lianZhongDao.getCollectNum(collectFlag);
        log.info("需要同步: " + collectNum + "条数据");
        // 计算需要轮询的次数
        int totalLoops = (collectNum + 999) / 1000; // 向上取整

        for (int i = 0; i < totalLoops; i++) {
            int taskNum = i;
            // 计算当前任务处理的数据范围
            int start = taskNum * 1000;
            int end = Math.min(start + 999, collectNum); // 确保不超过总条数
            //执行任务
            conllect(taskNum, start, end);
        }
    }

    /**
     * @description: 同步逻辑
     * @author linjj
     * @date: 2025/6/4 13:33
     */
    private void conllect(int taskNum, int start, int end) {
        log.info("轮询第 " + (taskNum + 1) + " 次，处理数据范围: " + start + " 到 " + end);
        log.info("采集状态为:" + collectFlag);
        //状态查询
        List<CardInfoVo> cardInfoVo = lianZhongDao.getCardInfoVo(start, end, collectFlag);
        if (ObjectUtils.isEmpty(cardInfoVo)) {
            log.info("当前没有需要同步的患者");
            return;
        }
        //每个患者进行轮询
        for (CardInfoVo list : cardInfoVo) {
            try {
                log.info("当前同步患者id为:" + list.getId());
                //查询患者当前文件信息
                List<Picture> pictures = lianZhongDao.getPicture(list.getId());
                if (ObjectUtils.isEmpty(pictures)) {
                    log.info("当前患者没有文件:" + list.getId());
                    continue;
                }
                //组织cmd命令
                for (Picture picture : pictures) {
                    try {
                        //保存目录
                        String dir = savePath + File.separatorChar + list.getId();
                        //保存地址
                        String savePath = dir + File.separatorChar + picture.getPicname();
                        // 创建File对象
                        File directory = new File(dir);
                        // 判断目录是否存在
                        if (!directory.exists()) {
                            // 目录不存在，创建目录
                            boolean created = directory.mkdirs();
                            if (created) {
                                log.info("目录创建成功：" + dir);
                            } else {
                                log.info("目录创建失败：" + dir);
                            }
                        }
                        //组织cmd命令(D:\js\Debug\Debug\lianzhong.exe为路径,10.1.1.128为联众弟子)
                        String cmd = "D:\\js\\Debug\\Debug\\lianzhong.exe 003 10.1.1.128 " + list.getId() + " " + picture.getPicid() + " " + list.getPatno() + " " + list.getOutdateStr() + " " + picture.getPicname() + " " + savePath + " " + uniUrl + " " + picture.getRotatedegree();
                        log.info("CMD:" + cmd);
                        Process process = Runtime.getRuntime().exec(cmd);//执行命令生成cube
                        process.waitFor();
                    } catch (Exception e) {
                        log.info("患者同步失败:" + list.getId());
                        log.error(e.getMessage(), e);
                    }
                }
            } catch (Exception e) {
                lianZhongDao.updateState(list.getId(), 3);
                log.info("患者同步失败:" + list.getId());
                log.error(e.getMessage(), e);
                continue;
            }
            lianZhongDao.updateState(list.getId(), 2);
            log.info("患者同步完成:" + list.getId());
        }
    }
}
